#!/bin/bash
#
# syntax         : checkrules.sh <mailfile>
#
# description    : The given mailfile is checked against the rules defined in
#                  your mailfilterrc. This script prints all matches and if
#                  the mail is deleted from the server and why.
#
#                  IMPORTANT NOTE: Set the variable MAILFILTERRC below to
#                                  point to your mailfilterrc file.
#
# author         : Kai Hildebrandt <kai.hildebrandt@onlinehome.de>
#
# date           : 2003-08-06
#
# version        : 0.3 (for mailfilter 0.5.1)
#
# license        : Licensed under the same terms as Mailfilter, GPL v2 or later.
# -----------------------------------------------------------------------------
# this variable should point to your mailfilterrc
# -----------------------------------------------------------------------------
MAILFILTERRC=$HOME/.mailfilterrc
# -----------------------------------------------------------------------------
# dont edit anything below this line
# -----------------------------------------------------------------------------
function usage()
{
    echo "usage: $( basename $0 ) [-M |--mailfilterrc=]<your mailfilterrc> <file>"
    exit 1
}

function get_options()
{
    if [ $# -eq 0 ]; then
	usage
    fi

    ARGC=0
    FLAG_RC=0

    for ARG in $*; do

	ARGC=$(( $ARGC + 1 ))

	if [ $FLAG_RC -ne 0 ]; then
	    MAILFILTERRC=$ARG
	    FLAG_RC=0
	    continue
	fi

	case $ARG in
	    -M) FLAG_RC=1
		;;

	    --mailfilterrc=*)
		MAILFILTERRC=$( echo $ARG | cut -d"=" -f2 )
		;;

	    *)
		if [ $ARGC -lt $# ]; then
		    echo "unknown option: $ARG"
		    usage
		else
		    MAILFILE=$ARG
		fi
		;;
	esac
    done
}

function clean_exit()
{
  # Remove tempfiles
    rm -f $TMPFILE
    rm -f $TMPFILE2

  # reset LANG
    LANG=$OLDLANG

    exit 0
}

# check options if any
get_options $*

if [ ! -f $MAILFILTERRC ]; then
    echo "error: $MAILFILTERRC not found."
    exit 2
fi

if [ ! -f $MAILFILE ]; then
    echo "error: $MAILFILE not found."
    exit 2
fi

# unset LANG
OLDLANG=$LANG
unset LANG

# tempfiles
TMPFILE=/var/tmp/$( basename $0 ).1.$$.tmp
TMPFILE2=/var/tmp/$( basename $0 ).2.$$.tmp

# make tempfiles unreadable by others
umask 077



# Catch SIGINT to clean up tempfiles at exit
trap clean_exit SIGINT



# get rules from mailfilterrc and included files
cat $MAILFILTERRC $( grep "^INCLUDE" $MAILFILTERRC | cut -f2 -d= ) | sed -e "s/#.*$//" | grep -E "^(DENY|ALLOW|SCORE|HIGHSCORE)" | sort -r > $TMPFILE

# search for self-defined HIGHSCORE
HIGHSCORE=$( grep "^HIGHSCORE" $TMPFILE | cut -f2 -d= )

# set default HIGHSCORE
if [ -z "$HIGHSCORE" ]; then
    HIGHSCORE=100
fi

# delete HIGHSCORE lines and substitute \ with \\ (\\\ ?)
sed -e "/^HIGHSCORE/d" -e "s/\\\/\\\\\\\/g" $TMPFILE > $TMPFILE2



# reset values
SCORE=0
ALLOWED=0
DENIED=0



# check all rules against file
while read LINE; do

    if [ ! -z "$( echo $LINE | grep = )" ]; then
	# equal rule
	WHAT=$( echo $LINE | cut -f1 -d= )
	PATTERN=$( echo $LINE | cut -f2- -d= )
	NOT=0
    else
	# not equal rule
	WHAT=$( echo $LINE | cut -f1 -d"<" )
	PATTERN=$( echo $LINE | cut -f2- -d"<" | sed -e "s/^.//" )
	NOT=1
    fi

    MATCHES=$( grep -E "$PATTERN" $MAILFILE | wc -l )

    if [ $NOT -eq 1 ]; then
	SHOWLINE="\"<>$PATTERN\""
    else
	SHOWLINE="\"$PATTERN\" to \"$( grep -E "$PATTERN" $MAILFILE )\""
    fi

    if [ $MATCHES -ge 1 -a $NOT -eq 0 -o $MATCHES -eq 0 -a $NOT -eq 1 ]; then

	case $WHAT in

	    ALLOW)
		echo "ALLOW rule match: $SHOWLINE"
		ALLOWED=1
		;;

	    DENY)
		echo "DENY rule match: $SHOWLINE"
		DENIED=1
		;;

	    SCORE*)
		echo "SCORE ($( echo $WHAT | cut -f2 -d" " )) rule match: $SHOWLINE"
		SCORE=$(( $SCORE $( echo $WHAT | cut -f2 -d" " ) ))
		;;
	esac
    fi

done < $TMPFILE2



echo
echo "Total score: $SCORE"
echo

if [ $ALLOWED -eq 1 ]; then
    echo "Mail would be downloaded from server because of an ALLOW rule."

elif [ $DENIED -eq 1 ]; then
    echo "Mail would be deleted from server because of a DENY rule."

elif [ $SCORE -ge $HIGHSCORE ]; then
    echo "Mail would be deleted from server because of reached HIGHSCORE."

else
    echo "Mail would be downloaded from server."

fi

clean_exit
